/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 17/04/22.
//

#include <fggl/gui/widget.hpp>
#include <fggl/gui/widgets.hpp>

#include <spdlog/spdlog.h>

namespace fggl::gui {

	Button::Button(math::vec2 pos, math::vec2 size) : Widget(pos, size), m_label(pos, size), m_hover(false),
													  m_active(false) {}

	void Button::render(gfx::Paint &paint) {
		gfx::Path2D path{topLeft()};
		draw_button(path, topLeft(), size(), m_hover, m_active);
		paint.fill(path);

		m_label.render(paint);
	}

	void Button::activate() {
		m_active = !m_active;
		if (m_active) {
			for (auto &callback : m_callbacks) {
				callback();
			}
			m_active = false;
		}
	}

	void Button::onEnter() {
		m_hover = true;
	}

	void Button::onExit() {
		m_hover = false;
	}

	void Button::addCallback(Callback cb) {
		m_callbacks.push_back(cb);
	}

	void Button::label(const std::string &value) {
		m_label.text(value);
	}

	std::string Button::label() const {
		return m_label.text();
	}

	void Label::layout() {
		if (m_font == nullptr) {
			return;
		}

		math::vec2 size;
		for (const auto &letter : m_value) {
			auto metrics = m_font->metrics(letter);
			size.x += (metrics.advance << 6);
			size.y = std::max(size.y, metrics.size.y);
		}
		m_naturalSize = size;
		m_needsLayout = false;
	}

	Label::Label(math::vec2 pos, math::vec2 size) : Widget(pos, size) {

	}
} // namespace fggl::gui